@@ -21,8 +21,6 @@ from django.utils.translation import ugettext_lazy as _
 
 from merengue.base.forms import BaseForm
 
-from cmsutils.db_utils import set_backup
-
 
 class BaseConfigForm(BaseForm):
     zipfile = forms.FileField(label=_("Select a .zip file:"))
@@ -39,12 +37,3 @@ class BaseConfigForm(BaseForm):
 
 class UploadConfigForm(BaseConfigForm):
     pass
-
-
-class BackupForm(BaseConfigForm):
-
-    def save(self):
-        zip_config = self.cleaned_data['zipfile']
-        f = zip_config.infolist()[0]
-        value = zip_config.read(f.filename)
-        set_backup(value)
