@@ -1,6 +1,7 @@
 from django.conf import settings
 from django.template.loader import render_to_string
-from django.template import RequestContext
+from django.template import RequestContext, defaultfilters
+from django.utils.translation import ugettext_lazy as _
 
 from copy import deepcopy
 from django.db import connection
@@ -48,3 +49,29 @@ def get_render_http_error(request, http_error):
     context = {'template_base': 'viewlet_error.html', 'is_admin': False}
     return render_to_string(template, context,
                                       context_instance=RequestContext(request))
+
+
+def get_login_url():
+    from merengue.pluggable.utils import get_plugin
+    core_config = get_plugin('core').get_config()
+    login_url_conf = core_config.get('login_url', None)
+    if not login_url_conf:
+        return settings.LOGIN_URL
+    else:
+        return login_url_conf.get_value() or settings.LOGIN_URL
+
+
+def get_unique_slug(value, queryset, slug_field='slug'):
+    """ Get the first empty slug in the queryset """
+    slug = original_slug = defaultfilters.slugify(value)
+    n = 2
+    filters = {slug_field: slug}
+    while queryset.filter(**filters).exists():
+        slug = original_slug + u'-%s' % n
+        filters[slug_field] = slug
+        n += 1
+    return slug
+
+
+def get_translate_status_list():
+    return ((status_code, _(status_text)) for status_code, status_text in settings.STATUS_LIST)
