@@ -1,4 +1,4 @@
-# Copyright (c) 2010 by Yaco Sistemas <msaelices@yaco.es>
+# Copyright (c) 2010 by Yaco Sistemas
 #
 # This file is part of Merengue.
 #
@@ -15,16 +15,22 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Merengue.  If not, see <http://www.gnu.org/licenses/>.
 
+from functools import wraps
 # django imports
+from django.conf import settings
+from django.db import connection
 from django.db.models import Q
-from django.contrib.auth.models import User
-from django.contrib.auth.models import Group
+from django.contrib.auth.models import User, AnonymousUser, Group
 from django.contrib.contenttypes.models import ContentType
+from django.core.cache import cache
 from django.core.exceptions import ObjectDoesNotExist, ImproperlyConfigured
 from django.template import defaultfilters
+from django.utils.decorators import available_attrs
 
-# permissions imports
-from merengue.perms import ANONYMOUS_ROLE_SLUG
+# merengue imports
+from merengue.cache import memoize, MemoizeCache
+from merengue.perms import ANONYMOUS_ROLE_SLUG, PARTICIPANT_ROLE_SLUG, OWNER_ROLE_SLUG
+from merengue.perms.exceptions import PermissionDenied
 from merengue.perms.models import ObjectPermission
 from merengue.perms.models import ObjectPermissionInheritanceBlock
 from merengue.perms.models import Permission
@@ -35,6 +41,95 @@ MANAGE_SITE_PERMISION = 'manage_site'
 MANAGE_USER_PERMISION = 'manage_user'
 MANAGE_MULTIMEDIA_PERMISSION = 'manage_multimedia'
 MANAGE_PLUGIN_PERMISSION = 'manage_plugin_content'
+MANAGE_BLOCK_PERMISSION = 'manage_block'
+MANAGE_CACHE_INVALIDATION_PERMISSION = 'cache_invalidation'
+
+
+# Cache stuff  #################################################################
+
+PERMS_CACHE_KEY = 'perms_cache'
+
+
+class RolesCache(MemoizeCache):
+
+    def clear(self, users_to_clear):
+        for k, v in self._cache.items():
+            if k[0] in users_to_clear:
+                del self._cache[k]
+        cache.set(self.cache_prefix, self._cache)
+
+_roles_cache = RolesCache('roles_cache')
+
+
+def clear_roles_cache(principal):
+    if isinstance(principal, User):
+        users_to_clear = [principal]
+    else:
+        users_to_clear = principal.user_set.all()
+    if users_to_clear:
+        _roles_cache.clear(users_to_clear)
+
+
+def get_from_cache(user, content, codename, roles):
+    if not settings.CACHE_PERMISSIONS:
+        return None
+    perms_cache = cache.get(PERMS_CACHE_KEY)
+    if perms_cache is None:
+        return None
+    roles_value = roles and tuple(r.id for r in roles) or None
+    return perms_cache.get(
+        (user.id, content, str(codename), roles_value),
+        None,
+    )
+
+
+def clear_cache():
+    cache.set(PERMS_CACHE_KEY, {})
+
+
+def set_permission_in_cache(global_perm=False):
+    """
+    Decorator for views makes store in cache after doing the logic
+    """
+    def decorator(view_func):
+        @wraps(view_func, assigned=available_attrs(view_func))
+        def _wrapped_func(*args, **kwargs):
+            has_permission = view_func(*args, **kwargs)
+            if isinstance(has_permission, tuple):
+                # a tuple (has_permission, "cached") has been returned
+                # this implies the permission was cached and there is no need
+                # to update cache again
+                return has_permission[0]
+            if not settings.CACHE_PERMISSIONS:
+                return has_permission
+            # update the permission cache
+            cache_values = {}
+            if global_perm:
+                cache_params = ('user', 'codename', 'roles')
+            else:
+                cache_params = ('content', 'user', 'codename', 'roles')
+            args_len = len(args)
+            for i, param in enumerate(cache_params):
+                if i < args_len:
+                    cache_values[param] = args[i]
+                else:  # is in kwargs
+                    cache_values[param] = kwargs.get(param, None)
+            if global_perm:
+                cache_values['content'] = None
+            perms_cache = cache.get(PERMS_CACHE_KEY, {})
+            roles = cache_values['roles'] and tuple(r.id for r in cache_values['roles']) or None
+            cache_key = (
+                getattr(cache_values['user'], 'id', None),
+                getattr(cache_values['content'], 'id', None),
+                str(cache_values['codename']),
+                roles,
+            )
+            perms_cache[cache_key] = has_permission
+            cache.set(PERMS_CACHE_KEY, perms_cache)
+            return has_permission
+        return _wrapped_func
+    return decorator
+
 
 # Roles ######################################################################
 
@@ -50,19 +145,20 @@ def add_role(principal, role):
     role
         The role which is assigned.
     """
+    clear_cache()
+    clear_roles_cache(principal)
     if isinstance(principal, User):
         try:
-            ppr = PrincipalRoleRelation.objects.get(user=principal, role=role, content__isnull=True)
+            PrincipalRoleRelation.objects.get(user=principal, role=role, content__isnull=True)
         except PrincipalRoleRelation.DoesNotExist:
             PrincipalRoleRelation.objects.create(user=principal, role=role)
             return True
     else:
         try:
-            ppr = PrincipalRoleRelation.objects.get(group=principal, role=role, content__isnull=True)
+            PrincipalRoleRelation.objects.get(group=principal, role=role, content__isnull=True)
         except PrincipalRoleRelation.DoesNotExist:
             PrincipalRoleRelation.objects.create(group=principal, role=role)
             return True
-
     return False
 
 
@@ -80,16 +176,17 @@ def add_local_role(obj, principal, role):
     role
         The role which is assigned.
     """
-    ctype = ContentType.objects.get_for_model(obj)
+    clear_cache()
+    clear_roles_cache(principal)
     if isinstance(principal, User):
         try:
-            ppr = PrincipalRoleRelation.objects.get(user=principal, role=role, content=obj)
+            PrincipalRoleRelation.objects.get(user=principal, role=role, content=obj)
         except PrincipalRoleRelation.DoesNotExist:
             PrincipalRoleRelation.objects.create(user=principal, role=role, content=obj)
             return True
     else:
         try:
-            ppr = PrincipalRoleRelation.objects.get(group=principal, role=role, content=obj)
+            PrincipalRoleRelation.objects.get(group=principal, role=role, content=obj)
         except PrincipalRoleRelation.DoesNotExist:
             PrincipalRoleRelation.objects.create(group=principal, role=role, content=obj)
             return True
@@ -108,6 +205,8 @@ def remove_role(principal, role):
     role
         The role which is removed.
     """
+    clear_cache()
+    clear_roles_cache(principal)
     try:
         if isinstance(principal, User):
             ppr = PrincipalRoleRelation.objects.get(
@@ -138,9 +237,9 @@ def remove_local_role(obj, principal, role):
     role
         The role which is removed.
     """
+    clear_cache()
+    clear_roles_cache(principal)
     try:
-        ctype = ContentType.objects.get_for_model(obj)
-
         if isinstance(principal, User):
             ppr = PrincipalRoleRelation.objects.get(
                 user=principal, role=role, content=obj)
@@ -164,6 +263,8 @@ def remove_roles(principal):
     principal
         The principal (user or group) from which all roles are removed.
     """
+    clear_cache()
+    clear_roles_cache(principal)
     if isinstance(principal, User):
         ppr = PrincipalRoleRelation.objects.filter(
             user=principal, content__isnull=True)
@@ -190,8 +291,8 @@ def remove_local_roles(obj, principal):
     principal
         The principal (user or group) from which the roles are removed.
     """
-    ctype = ContentType.objects.get_for_model(obj)
-
+    clear_cache()
+    clear_roles_cache(principal)
     if isinstance(principal, User):
         ppr = PrincipalRoleRelation.objects.filter(
             user=principal, content=obj)
@@ -206,7 +307,7 @@ def remove_local_roles(obj, principal):
         return False
 
 
-def get_roles(principal, obj=None):
+def _get_roles(user, obj=None):
     """Returns all roles of passed user for passed content object. This takes
     direct and roles via a group into account. If an object is passed local
     roles will also added.
@@ -214,28 +315,74 @@ def get_roles(principal, obj=None):
     **Parameters:**
 
     obj
-        The object from which the roles are removed.
+        The object from which the roles are returned.
 
-    principal
-        The principal (user or group) from which the roles are removed.
+    user
+        The user from which the roles are returned.
     """
-    roles = get_global_roles(principal)
+    if isinstance(user, AnonymousUser):
+        return Role.objects.filter(slug=ANONYMOUS_ROLE_SLUG)
 
-    if obj is not None:
-        roles.extend(get_local_roles(obj, principal))
+    role_ids = []
+    groups = user.groups.all()
+    groups_ids_str = ", ".join([str(g.id) for g in groups])
 
-    if isinstance(principal, User):
-        for group in principal.groups.all():
-            if obj is not None:
-                roles.extend(get_local_roles(obj, group))
-            roles.extend(get_roles(group))
+    # Gobal roles for user and the user's groups
+    cursor = connection.cursor()
 
-    return roles
+    if groups_ids_str:
+        cursor.execute("""SELECT role_id
+                          FROM perms_principalrolerelation
+                          WHERE (user_id=%s OR group_id IN (%s))
+                          AND content_id is Null""" % (user.id, groups_ids_str))
+    else:
+        cursor.execute("""SELECT role_id
+                          FROM perms_principalrolerelation
+                          WHERE user_id=%s
+                          AND content_id is Null""" % user.id)
+
+    for row in cursor.fetchall():
+        role_ids.append(row[0])
+
+    # Local roles for user and the user's groups and all ancestors of the
+    # passed object.
+    while obj:
+        if groups_ids_str:
+            cursor.execute("""SELECT role_id
+                              FROM perms_principalrolerelation
+                              WHERE (user_id='%s' OR group_id IN (%s))
+                              AND content_id='%s'""" % (user.id, groups_ids_str, obj.id))
+        else:
+            cursor.execute("""SELECT role_id
+                              FROM perms_principalrolerelation
+                              WHERE user_id='%s'
+                              AND content_id='%s'""" % (user.id, obj.id))
+
+        for row in cursor.fetchall():
+            role_ids.append(row[0])
+
+        get_owners = getattr(obj, 'get_owners', None)
+        get_participants = getattr(obj, 'get_participants', None)
+        if get_owners and callable(get_owners) and user in obj.get_owners():
+            role_ids.append(Role.objects.get(slug=OWNER_ROLE_SLUG).id)
+
+        if get_participants and callable(get_participants) and user in obj.get_participants():
+            role_ids.append(Role.objects.get(slug=PARTICIPANT_ROLE_SLUG).id)
+
+        try:
+            obj = obj.get_parent_for_permissions()
+        except AttributeError:
+            obj = None
+
+    return Role.objects.filter(pk__in=role_ids)
+get_roles = memoize(_get_roles, _roles_cache, 2)
 
 
 def get_global_roles(principal):
     """Returns global roles of passed principal (user or group).
     """
+    if isinstance(principal, AnonymousUser):
+        return Role.objects.filter(slug=ANONYMOUS_ROLE_SLUG)
     if isinstance(principal, User):
         return [prr.role for prr in PrincipalRoleRelation.objects.filter(
             user=principal, content__isnull=True)]
@@ -247,16 +394,39 @@ def get_global_roles(principal):
 def get_local_roles(obj, principal):
     """Returns local for passed user and content object.
     """
-    ctype = ContentType.objects.get_for_model(obj)
-
     if isinstance(principal, User):
-        return [prr.role for prr in PrincipalRoleRelation.objects.filter(
+        roles = [prr.role for prr in PrincipalRoleRelation.objects.filter(
             user=principal, content=obj)]
+
+        if principal in obj.get_owners():
+            roles.append(Role.objects.get(slug=OWNER_ROLE_SLUG))
+
+        if principal in obj.get_participants():
+            roles.append(Role.objects.get(slug=PARTICIPANT_ROLE_SLUG))
+        return roles
     else:
         return [prr.role for prr in PrincipalRoleRelation.objects.filter(
             group=principal, content=obj)]
 
 
+def get_all_local_roles(obj):
+    roles = []
+    for relation in PrincipalRoleRelation.objects.filter(content=obj):
+        if relation.user:
+            roles.append((relation.user, relation.role))
+        elif relation.group:
+            roles.append((relation.group, relation.role))
+        else:
+            continue
+    owner_role = Role.objects.get(slug=OWNER_ROLE_SLUG)
+    for principal in obj.get_owners():
+        roles.append((principal, owner_role))
+    participant_role = Role.objects.get(slug=PARTICIPANT_ROLE_SLUG)
+    for principal in obj.get_participants():
+        roles.append((principal, participant_role))
+    return roles
+
+
 # Permissions ################################################################
 
 
@@ -278,15 +448,13 @@ def grant_permission(role, permission, obj=None):
         If obj is None, the permissions should be granted for all contents.
 
     """
+    clear_cache()
     if not isinstance(permission, Permission):
         try:
-            permission = Permission.objects.get(codename = permission)
+            permission = Permission.objects.get(codename=permission)
         except Permission.DoesNotExist:
             return False
 
-    if obj:
-        ct = ContentType.objects.get_for_model(obj)
-
     try:
         ObjectPermission.objects.get(role=role, content=obj, permission=permission)
     except ObjectPermission.DoesNotExist:
@@ -312,17 +480,15 @@ def remove_permission(role, permission, obj=None):
         The permission which should be removed. Either a permission object
         or the codename of a permission.
     """
+    clear_cache()
     if not isinstance(permission, Permission):
         try:
-            permission = Permission.objects.get(codename = permission)
+            permission = Permission.objects.get(codename=permission)
         except Permission.DoesNotExist:
             return False
 
-    if obj:
-        ct = ContentType.objects.get_for_model(obj)
-
     try:
-        op = ObjectPermission.objects.get(role=role, content=obj, permission = permission)
+        op = ObjectPermission.objects.get(role=role, content=obj, permission=permission)
     except ObjectPermission.DoesNotExist:
         return False
 
@@ -330,6 +496,7 @@ def remove_permission(role, permission, obj=None):
     return True
 
 
+@set_permission_in_cache(global_perm=True)
 def has_global_permission(user, codename, roles=None):
     """Checks whether the passed user has passed permission for passed object.
 
@@ -346,17 +513,26 @@ def has_global_permission(user, codename, roles=None):
         the permissions are checked.
     """
 
-    if roles is None:
-        roles = []
-
-    roles.append(Role.objects.get(slug=ANONYMOUS_ROLE_SLUG))
-
     if user.is_superuser:
         return True
 
-    if user.is_anonymous():
-        user = None
-    else:
+    # try to get from cache
+    cached_perm = get_from_cache(user, None, codename, roles)
+    if cached_perm is not None:
+        return cached_perm, "cached"  # latest returned value never will reach the function which makes call
+
+    if roles is None:
+        roles = []
+
+    # we check first ANONYMOUS_ROLE_SLUG to reduce SQL sentences and improve
+    # cache hit percentage (next sentence is more susceptible to be cached)
+    p_anon = ObjectPermission.objects.filter(
+        Q(content__isnull=True, role__slug=ANONYMOUS_ROLE_SLUG, permission__codename=codename))
+
+    if p_anon:
+        return True
+
+    if not user.is_anonymous():
         roles.extend(get_roles(user))
 
     p = ObjectPermission.objects.filter(
@@ -368,6 +544,7 @@ def has_global_permission(user, codename, roles=None):
     return False
 
 
+@set_permission_in_cache()
 def has_permission(obj, user, codename, roles=None):
     """Checks whether the passed user has passed permission for passed object.
 
@@ -387,28 +564,47 @@ def has_permission(obj, user, codename, roles=None):
         the permissions are checked.
     """
 
+    if user.is_superuser:
+        return True
+
+    # try to get from cache
+    cached_perm = get_from_cache(user, obj, codename, roles)
+    if cached_perm is not None:
+        return cached_perm, "cached"  # latest returned value never will reach the function which makes call
+
     if roles is None:
         roles = []
 
     roles.append(Role.objects.get(slug=ANONYMOUS_ROLE_SLUG))
 
-    if user.is_superuser:
-        return True
+    global_perm = has_global_permission(user, codename, roles)
+    if not obj or (obj and getattr(obj, 'adquire_global_permissions', True) and global_perm):
+        # returns global_perm if there is not an object or there is an object
+        # which adquire global permissions and global_perm is True and there is
+        # not need to further processing
+        return global_perm
 
-    if not obj:
-        return has_global_permission(user, codename, roles)
+    if not can_use_permissions(obj):
+        # object does not support permissions
+        return False
 
     if user.is_anonymous():
         user = None
     else:
         roles.extend(get_roles(user, obj))
 
-    ct = ContentType.objects.get_for_model(obj)
-
     while obj is not None:
-        p = ObjectPermission.objects.filter(
-            Q(content=obj, role__in=roles, permission__codename=codename) |
-            Q(content__isnull=True, role__in=roles, permission__codename=codename))
+        # if obj doesn't support permissions, use related basecontent
+        if getattr(obj, 'adquire_global_permissions', None) is None:
+            try:
+                obj = obj.basecontent_set.all()[0]
+            except AttributeError:
+                return False
+            except IndexError:
+                return False
+
+        filters = Q(content=obj, role__in=roles, permission__codename=codename)
+        p = ObjectPermission.objects.filter(filters)
 
         if p:
             return True
@@ -418,7 +614,6 @@ def has_permission(obj, user, codename, roles=None):
 
         try:
             obj = obj.get_parent_for_permissions()
-            ct = ContentType.objects.get_for_model(obj)
         except AttributeError:
             return False
 
@@ -449,6 +644,30 @@ def has_permission_in_queryset(queryset, user, codename, roles=None):
     return True
 
 
+def assert_has_permission(obj, user, codename, roles=None):
+    if not has_permission(obj, user, codename, roles):
+        raise PermissionDenied(content=obj, user=user, perm=codename)
+    return True
+
+
+def assert_has_global_permission(user, codename, roles=None):
+    if not has_global_permission(user, codename, roles):
+        raise PermissionDenied(user=user, perm=codename)
+    return True
+
+
+def assert_has_permission_in_queryset(queryset, user, codename, roles=None):
+    for obj in queryset:
+        if not has_permission(obj, user, codename, roles):
+            raise PermissionDenied(content=obj, user=user, perm=codename)
+    return True
+
+
+def can_use_permissions(obj):
+    """ The obj can contains permissions """
+    from merengue.base.models import BaseContent
+    return isinstance(obj, BaseContent)
+
 # Inheritance ################################################################
 
 
@@ -465,11 +684,10 @@ def add_inheritance_block(obj, permission):
     """
     if not isinstance(permission, Permission):
         try:
-            permission = Permission.objects.get(codename = permission)
+            permission = Permission.objects.get(codename=permission)
         except Permission.DoesNotExist:
             return False
 
-    ct = ContentType.objects.get_for_model(obj)
     try:
         ObjectPermissionInheritanceBlock.objects.get(content=obj, permission=permission)
     except ObjectPermissionInheritanceBlock.DoesNotExist:
@@ -494,11 +712,10 @@ def remove_inheritance_block(obj, permission):
     """
     if not isinstance(permission, Permission):
         try:
-            permission = Permission.objects.get(codename = permission)
+            permission = Permission.objects.get(codename=permission)
         except Permission.DoesNotExist:
             return False
 
-    ct = ContentType.objects.get_for_model(obj)
     try:
         opi = ObjectPermissionInheritanceBlock.objects.get(content=obj, permission=permission)
     except ObjectPermissionInheritanceBlock.DoesNotExist:
@@ -520,10 +737,9 @@ def is_inherited(obj, codename):
             The permission which should be checked. Must be the codename of
             the permission.
     """
-    ct = ContentType.objects.get_for_model(obj)
     try:
         ObjectPermissionInheritanceBlock.objects.get(
-            content=obj, permission__codename = codename)
+            content=obj, permission__codename=codename)
     except ObjectDoesNotExist:
         return True
     else:
@@ -546,6 +762,30 @@ def can_manage_plugin_content(user):
     return has_global_permission(user, MANAGE_PLUGIN_PERMISSION)
 
 
+def assert_manage_site(user):
+    if not has_global_permission(user, MANAGE_SITE_PERMISION):
+        raise PermissionDenied(user=user, perm=MANAGE_SITE_PERMISION)
+    return True
+
+
+def assert_manage_user(user):
+    if not has_global_permission(user, MANAGE_USER_PERMISION):
+        raise PermissionDenied(user=user, perm=MANAGE_SITE_PERMISION)
+    return True
+
+
+def assert_manage_multimedia(user):
+    if not has_global_permission(user, MANAGE_MULTIMEDIA_PERMISSION):
+        raise PermissionDenied(user=user, perm=MANAGE_SITE_PERMISION)
+    return True
+
+
+def assert_manage_plugin_content(user):
+    if not has_global_permission(user, MANAGE_PLUGIN_PERMISSION):
+        raise PermissionDenied(user=user, perm=MANAGE_SITE_PERMISION)
+    return True
+
+
 def get_group(id):
     """Returns the group with passed id or None.
     """
@@ -576,7 +816,6 @@ def get_user(id):
 def reset(obj):
     """Resets all permissions and inheritance blocks of passed object.
     """
-    ctype = ContentType.objects.get_for_model(obj)
     ObjectPermissionInheritanceBlock.objects.filter(content=obj).delete()
     ObjectPermission.objects.filter(content=obj).delete()
 
