@@ -18,16 +18,17 @@
 from django.conf import settings
 from django.contrib import admin
 from django.db.models import Q
+from django import forms
+from django.forms.util import ValidationError
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext_lazy as _
 from django.utils.translation import ugettext
 
-from transmeta import get_fallback_fieldname
+from ajax_select.fields import AutoCompleteSelectField
+from transmeta import get_real_fieldname_in_each_language, get_fallback_fieldname
 
-from merengue.base.admin import BaseAdmin, BaseContentAdmin, RelatedModelAdmin, \
-                                BaseOrderableAdmin, OrderableRelatedModelAdmin
-from merengue.base.admin import set_field_read_only
-from merengue.base.widgets import RelatedFieldWidgetWrapperWithoutAdding
+from merengue.base.admin import (BaseAdmin, BaseContentAdmin,
+                                 RelatedModelAdmin, OrderableRelatedModelAdmin)
 from merengue.section.fields import CSSValidatorField
 from merengue.section.forms import MenuAdminModelForm
 from merengue.section.models import (Menu, BaseSection,
@@ -39,14 +40,37 @@ from merengue.perms import utils as perms_api
 from merengue.perms.admin import PermissionAdmin
 
 
-class BaseSectionAdmin(BaseOrderableAdmin, PermissionAdmin):
+class BaseSectionAdmin(BaseContentAdmin, PermissionAdmin):
     sortablefield = 'order'
     ordering = ('order', )
-    list_display = ('name', 'slug', )
-    list_display_links = ('name', )
+    list_display = BaseContentAdmin.list_display[:-1]
+    search_fields = tuple(get_real_fieldname_in_each_language('name'))
     html_fields = ()
     removed_fields = ('description', )
     prepopulated_fields = {'slug': (get_fallback_fieldname('name'), )}
+    exclude = BaseContentAdmin.exclude + ('commentable', )
+
+    def get_object(self, request, object_id):
+        """
+        Overrides the django behaviour
+        """
+        queryset = self.queryset(request, bypass_perms=True)
+        model = queryset.model
+        try:
+            object_id = model._meta.pk.to_python(object_id)
+            return queryset.get(pk=object_id)
+        except (model.DoesNotExist, ValidationError):
+            return None
+
+    def queryset(self, request, bypass_perms=False):
+        """
+        Overrides the Django behaviour to take permissions into account
+        """
+        qs = super(BaseSectionAdmin, self).queryset(request)
+        if not bypass_perms and not perms_api.can_manage_site(request.user) and \
+           not perms_api.has_global_permission(request.user, 'edit'):
+            qs = qs.filter(Q(owners=request.user))
+        return qs
 
     def has_add_permission(self, request):
         """
@@ -58,11 +82,13 @@ class BaseSectionAdmin(BaseOrderableAdmin, PermissionAdmin):
         """
         Overrides Django admin behaviour to add ownership based access control
         """
+        permission = super(BaseSectionAdmin, self).has_change_permission(request, obj)
+        if permission:
+            return permission
         if perms_api.has_global_permission(request.user, 'manage_section'):
             return True
-        else:
-            if BaseSection.objects.filter(owners=request.user).count():
-                return True
+        elif obj is None:
+            return True
         return False
 
     def has_delete_permission(self, request, obj=None):
@@ -71,15 +97,6 @@ class BaseSectionAdmin(BaseOrderableAdmin, PermissionAdmin):
         """
         return perms_api.has_global_permission(request.user, 'manage_section')
 
-    def queryset(self, request):
-        qs = super(BaseSectionAdmin, self).queryset(request)
-        if perms_api.has_global_permission(request.user, 'manage_portal'):
-            return qs
-        elif self.has_change_permission(request):
-            return qs.filter(Q(owners=request.user))
-        else:
-            return qs.none()
-
     def get_form(self, request, obj=None, **kwargs):
         form = super(BaseSectionAdmin, self).get_form(request, obj, **kwargs)
         if 'main_content' in form.base_fields.keys():
@@ -115,6 +132,19 @@ class ViewletLinkAdmin(BaseAdmin):
 class SectionAdmin(BaseSectionAdmin):
 
     change_list_template = 'admin/section/section/change_list.html'
+    change_form_template = 'admin/section/section/change_form.html'
+
+    def get_form(self, request, obj=None, **kwargs):
+        form = super(SectionAdmin, self).get_form(request, obj, **kwargs)
+        if 'section' in form.base_fields:
+            del form.base_fields['section']
+        if 'main_content' in form.base_fields.keys():
+            link_autocomplete = AutoCompleteSelectField(
+                'section_contentlink', label=ugettext('main content'),
+            )
+            link_autocomplete.widget.help_text = ugettext('content selected here will be shown when entering the section')
+            form.base_fields['main_content'] = link_autocomplete
+        return form
 
     def render_change_form(self, request, context, add=False, change=False, form_url='', obj=None):
         context['change_form_section'] = True
@@ -141,6 +171,14 @@ class SectionContentAdmin(OrderableRelatedModelAdmin):
     sortablefield = 'order'
     manage_contents = True
 
+    def get_form(self, request, obj=None, **kwargs):
+        form = super(SectionContentAdmin, self).get_form(request, obj, **kwargs)
+        if 'section' in form.base_fields:
+            # the section should be not editable because the content
+            # is in a admin related to this section. we remove the field
+            del form.base_fields['section']
+        return form
+
     def custom_relate_content(self, request, obj, form, change):
         if not change:
             SectionRelatedContent.objects.create(
@@ -176,8 +214,7 @@ class CustomStyleRelatedModelAdmin(RelatedModelAdmin):
 
 
 class DocumentAdmin(BaseContentAdmin):
-    list_display = ('name', 'slug', 'status', )
-    list_filter = ('status', )
+    list_display = ('name', 'slug', 'workflow_status', )
     html_fields = ('description', 'body', )
     prepopulated_fields = {'slug': (get_fallback_fieldname('name'), )}
     actions = BaseAdmin.actions + ['set_as_published', 'set_as_draft']
@@ -188,15 +225,10 @@ class DocumentRelatedModelAdmin(SectionContentAdmin, DocumentAdmin):
     tool_label = _('documents')
     manage_contents = True
 
-    def get_form(self, request, obj=None, **kwargs):
-        form = super(DocumentRelatedModelAdmin, self).get_form(request, obj, **kwargs)
-        if obj and obj.permanent and 'slug' in form.base_fields.keys():
-            set_field_read_only(form.base_fields['slug'], 'slug', obj)
-        return form
-
 
 class BaseLinkInline(admin.TabularInline):
 
+    extra = 0
     formset = BaseLinkInlineFormSet
 
     def get_fieldsets(self, request, obj=None):
@@ -223,13 +255,11 @@ class BaseLinkInline(admin.TabularInline):
 class AbsoluteLinkInline(BaseLinkInline):
     model = AbsoluteLink
     max_num = 1
+    extra = 1
     verbose_name = _('Menu Absolute Link')
     verbose_name_plural = _('Menu Absolute Links')
 
     def _media(self):
-        from django import forms
-
-        #js = ['%sjs/tiny_mce_internal_links/no_tiny_mce_internal_links.js' % settings.MEDIA_URL]
         js = []
         if self.prepopulated_fields:
             js.append('%sjs/urlify.js' % settings.ADMIN_MEDIA_PREFIX)
@@ -253,6 +283,7 @@ class AbsoluteLinkInline(BaseLinkInline):
 class ContentLinkInline(BaseLinkInline):
     model = ContentLink
     max_num = 1
+    extra = 1
     verbose_name = _('Menu Content Link')
     verbose_name_plural = _('Menu Content Links')
 
@@ -262,19 +293,11 @@ class ContentLinkInline(BaseLinkInline):
             return formset
         form = formset.form
         if 'content' in form.base_fields.keys():
-            formfield = form.base_fields['content']
-            qs = formfield.queryset
-            admin_model = getattr(self, 'admin_model', None)
-            if admin_model:
-                qs = qs.filter(sections=admin_model.basecontent)
-            formfield.queryset = qs
-            # change the widget to a wrapper
-            # note: formfield.wigdet is already a wrapper of formfield.widget.widget
-            formfield.widget = RelatedFieldWidgetWrapperWithoutAdding(
-                formfield.widget.widget, formfield.widget.rel, self.admin_site,
+            link_autocomplete = AutoCompleteSelectField(
+                'section_contentlink', label=ugettext('Content'),
             )
-            # limiting widget choices to new queryset
-            formfield.widget.widget.choices = formfield.choices
+            link_autocomplete.widget.help_text = ugettext('Enter text to search the content')
+            form.base_fields['content'] = link_autocomplete
         return formset
 
 
@@ -296,6 +319,12 @@ class MenuAdmin(BaseAdmin):
     form = MenuAdminModelForm
     menu_slug = None
 
+    class Media:
+        css = {'all': ('merengue/css/ajaxautocompletion/jquery.autocomplete.css',
+                       'merengue/css/ajax_select/iconic.css')}
+        js = ('merengue/js/ajaxautocompletion/jquery.autocomplete.js',
+              'merengue/js/ajax_select/ajax_select.js')
+
     def __init__(self, *args, **kwargs):
         super(MenuAdmin, self).__init__(*args, **kwargs)
         self.old_inline_instances = [instance for instance in self.inline_instances]
@@ -372,9 +401,7 @@ class MenuAdmin(BaseAdmin):
     def changelist_view(self, request, extra_context={}):
         source = self.move_menus(request)
         media = self.media
-        media.add_js([settings.MEDIA_URL + "merengue/js/jquery-ui-1.8.dragdrop.min.js"])
         media.add_js([settings.MEDIA_URL + "merengue/js/section/CollapsableMenuTree.js"])
-        media.add_js([settings.MEDIA_URL + "merengue/js/section/OrderableMenuTree.js"])
         extra_context.update({'media': media.render(),
                               'moved_source': source})
         return super(MenuAdmin, self).changelist_view(request, extra_context)
@@ -383,13 +410,23 @@ class MenuAdmin(BaseAdmin):
 class BaseSectionMenuRelatedAdmin(MenuAdmin, RelatedModelAdmin):
     change_list_template = "admin/section/menu/change_list.html"
 
+    def has_add_permission(self, request):
+        return RelatedModelAdmin.has_add_permission(self, request)
+
+    def has_change_permission(self, request, obj=None):
+        return RelatedModelAdmin.has_change_permission(self, request, obj)
+
+    def has_delete_permission(self, request, obj=None):
+        return RelatedModelAdmin.has_delete_permission(self, request, obj)
+
     def get_section_docs(self, request):
         section = getattr(self.get_menu(request), self.related_field)
         return section.content_set.all()
 
     def get_menu(self, request, *args, **kwargs):
         basecontent = kwargs.pop('basecontent', None)
-        return RelatedModelAdmin.queryset(self, request, basecontent).get()
+        queryset = RelatedModelAdmin.queryset(self, request, basecontent)
+        return queryset and queryset.get() or None
 
     def get_form(self, request, obj=None, **kwargs):
         form = super(BaseSectionMenuRelatedAdmin, self).get_form(request, obj, **kwargs)
