@@ -16,7 +16,6 @@
 # along with Merengue.  If not, see <http://www.gnu.org/licenses/>.
 
 from django.conf import settings
-from django.contrib.auth.decorators import login_required
 from django.http import HttpResponse, HttpResponseRedirect, Http404
 from django.shortcuts import render_to_response, get_object_or_404
 from django.template import RequestContext
@@ -25,13 +24,12 @@ from django.utils.translation import get_language_from_request
 
 from merengue.section.models import BaseSection, Document, \
                                     DocumentSection, BaseLink
-
+from merengue.base.decorators import login_required
 from merengue.base.views import content_view
 from merengue.section.models import AbsoluteLink, ContentLink, ViewletLink, Menu
 from merengue.section.utils import get_section
 
 from merengue.perms import utils as perms_api
-from merengue.base.utils import get_render_http_error
 
 
 def section_index(request):
@@ -43,11 +41,8 @@ def section_view(request, section_slug, original_context={},
                  template='section/section_view_without_maincontent.html'):
     section_slug = section_slug.strip('/')
     section = get_object_or_404(BaseSection, slug=section_slug)
-    has_view = perms_api.has_permission(section, request.user, 'view')
-    if not has_view:
-        forbidden_response = HttpResponse()
-        forbidden_response.write(get_render_http_error(request, 403))
-        return forbidden_response
+
+    perms_api.assert_has_permission(section, request.user, 'view')
     context = original_context or {}
     context['section'] = section.get_real_instance()
     main_content = section.main_content and section.main_content.get_real_instance() or None
@@ -81,7 +76,7 @@ def menu_section_view(request, section_slug, menu_slug):
     else:
         root_menu = Menu.objects.get(slug=settings.MENU_PORTAL_SLUG)
     try:
-        menu = root_menu.get_descendants().get(slug=menu_slug)
+        menu = root_menu.get_descendants_by_user(request.user).get(slug=menu_slug)
     except Menu.DoesNotExist:
         try:
             if not section_slug:
@@ -96,10 +91,18 @@ def menu_section_view(request, section_slug, menu_slug):
     try:
         link = menu.baselink.real_instance
     except BaseLink.DoesNotExist:
+        can_edit = False
+        if section:
+            can_edit = section.can_edit(request.user)
+        else:
+            can_edit = menu.can_edit(request.user)
         return render_to_response('section/menu_link_not_exists.html',
-            {'menu': menu}, context_instance=RequestContext(request))
+            {'menu': menu, 'can_edit': can_edit}, context_instance=RequestContext(request))
     if isinstance(link, AbsoluteLink):
-        return HttpResponseRedirect(link.get_absolute_url())
+        url_redirect = link.get_absolute_url()
+        if  url_redirect != request.get_full_path():
+            return HttpResponseRedirect(url_redirect)
+        raise Http404
     else:
         context = {}
         if section_slug:
@@ -125,11 +128,8 @@ def section_view_without_maincontent(request, context,
     user = request.user
     section = context['section']
     admin_absolute_url = False
-    if user.is_authenticated():
-        if user.is_superuser or (user.is_staff and
-                                 (user.has_perm('section.change_%s' % section._meta.module_name) or
-                                  user.has_perm('section.change_basesection'))):
-            admin_absolute_url = True
+    if section.can_edit(user):
+        admin_absolute_url = True
     context['admin_absolute_url'] = admin_absolute_url
     return render_to_response(template, context,
                               context_instance=RequestContext(request))
@@ -166,6 +166,8 @@ def section_dispatcher(request, url):
     kwargs = dict(section_slug=parts[0])
     func = None
 
+    if url.startswith(settings.MEDIA_URL):
+        raise Http404  # shortcut to avoid uneeded cpu cycles
     if len(parts) == 1:
         func = section_view
     elif len(parts) == 4 and parts[1] == 'contents':
