@@ -15,21 +15,19 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Merengue.  If not, see <http://www.gnu.org/licenses/>.
 
+from django.db import models
 
-from django.conf import settings
-
-if settings.USE_GIS:
-    from django.contrib.gis.db import models
-else:
-    from django.db import models
-
-from django.contrib.admin import widgets
 from django.forms.util import ValidationError
-from django.forms.fields import email_re
+from django.core.validators import email_re
 from django.utils.translation import ugettext as _
 from django import forms
 
-from plugins.contactform.recaptcha.client import captcha
+from merengue.base import widgets
+from merengue.pluggable.utils import get_plugin
+
+from plugins.contactform.utils import captcha_displayajaxhtml
+
+from recaptcha.client import captcha
 
 
 class RadioField(forms.ChoiceField):
@@ -41,11 +39,11 @@ class TextAreaField(forms.CharField):
 
 
 class DateTimeField(forms.DateTimeField):
-    widget = widgets.AdminSplitDateTime
+    widget = widgets.SplitDateTimeWidget
 
 
 class DateField(forms.DateField):
-    widget = widgets.AdminDateWidget
+    widget = widgets.InputDateWidget
 
 
 class MultiEmailField(forms.CharField):
@@ -76,8 +74,8 @@ class ModelMultiEmailField(models.TextField):
 class CaptchaWidget(forms.widgets.Widget):
 
     def render(self, name, value, attrs=None):
-        from plugins.contactform.config import PluginConfig
-        pubkey = PluginConfig.get_config().get('rpubk', None)
+        plugin = get_plugin('contactform')
+        pubkey = plugin.get_config().get('rpubk', None)
         if pubkey:
             pubkey = pubkey.value
         else:
@@ -97,8 +95,8 @@ class CaptchaField(forms.Field):
         super(CaptchaField, self).__init__(*args, **kwargs)
 
     def clean(self, value):
-        from plugins.contactform.config import PluginConfig
-        privkey = PluginConfig.get_config().get('rprivk', None)
+        plugin = get_plugin('contactform')
+        privkey = plugin.get_config().get('rprivk', None)
         if privkey:
             privkey = privkey.value
         else:
@@ -108,3 +106,19 @@ class CaptchaField(forms.Field):
                                             privkey, self.client_addr)
         if not recaptcha_response.is_valid:
             raise ValidationError(_("The captcha was invalid, try again."))
+
+
+class CaptchaAjaxWidget(CaptchaWidget):
+
+    def render(self, name, value, attrs=None):
+        plugin = get_plugin('contactform')
+        pubkey = plugin.get_config().get('rpubk', None)
+        if pubkey:
+            pubkey = pubkey.value
+        else:
+            pubkey = ''
+        return captcha_displayajaxhtml(pubkey)
+
+
+class CaptchaAjaxField(CaptchaField):
+    widget = CaptchaAjaxWidget
