@@ -21,7 +21,6 @@ from transmeta import get_fallback_fieldname
 
 from merengue.base.admin import BaseContentAdmin, BaseCategoryAdmin
 from merengue.section.admin import SectionContentAdmin
-from plugins.event.models import Event, Category
 from plugins.event.forms import EventAdminForm
 
 
@@ -33,18 +32,13 @@ class EventCategoryAdmin(BaseCategoryAdmin):
 class EventAdmin(BaseContentAdmin):
     ordering = (get_fallback_fieldname('name'), )
     exclude = ('expire_date', )
-    list_display = ('name', 'start', 'end', 'status', 'user_modification_date',
+    list_display = ('name', 'start', 'end', 'workflow_status', 'user_modification_date',
                     'last_editor')
     form = EventAdminForm
     if settings.USE_GIS:
         list_display = list_display + ('google_minimap', )
-    list_filter = ('categories', 'status', 'user_modification_date', )
+    list_filter = ('categories', 'workflow_status', 'user_modification_date', )
 
 
 class EventSectionAdmin(SectionContentAdmin, EventAdmin):
     manage_contents = True
-
-
-def register(site):
-    site.register(Category, EventCategoryAdmin)
-    site.register(Event, EventAdmin)
