@@ -20,10 +20,10 @@ from django.core import urlresolvers
 from django.http import HttpResponseRedirect
 from django.shortcuts import get_object_or_404
 
+from merengue.perms import utils as perms_api
+from merengue.section.views import section_view, content_section_view
 
 from plugins.microsite.models import MicroSite
-from merengue.section.models import Document
-from merengue.section.views import section_view, content_section_view
 
 
 def microsite_view(request, microsite_slug):
@@ -32,6 +32,7 @@ def microsite_view(request, microsite_slug):
 
 def microsite_url(request, microsite_slug, url):
     microsite = get_object_or_404(MicroSite, slug=microsite_slug)
+    perms_api.assert_has_permission(microsite, request.user, 'view')
     urlconf = getattr(request, "urlconf", settings.ROOT_URLCONF)
     urlresolvers.set_urlconf(urlconf)
     index_prefix = request.get_full_path().index(microsite_slug)
@@ -63,6 +64,5 @@ def microsite_url(request, microsite_slug, url):
     return callback(request, *callback_args, **callback_kwargs)
 
 
-def document_microsite_view(request, microsite_slug, doc_slug):
-    doc = get_object_or_404(Document, slug=doc_slug)
-    return content_section_view(request, microsite_slug, doc.id, doc_slug)
+def document_microsite_view(request, microsite_slug, doc_id, doc_slug):
+    return content_section_view(request, microsite_slug, doc_id, doc_slug)
